/*
 *  Copyright Droids Corporation (2011)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: commands.c,v 1.9 2009-11-08 17:24:33 zer0 Exp $
 *
 *  Olivier MATZ <zer0@droids-corp.org>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <aversive.h>
#include <aversive/pgmspace.h>
#include <aversive/queue.h>
#include <aversive/endian.h>
#include <aversive/error.h>
#include <aversive/wait.h>
#include <parse.h>
#include <rdline.h>
#include <parse_string.h>
#include <parse_num.h>
#include <uart.h>
#include <xbee.h>
#include <callout.h>

#include "main.h"
#include "cmdline.h"
#include "eeprom_config.h"

/* commands_gen.c */
extern const parse_inst_t PROGMEM cmd_reset;
extern const parse_inst_t PROGMEM cmd_bootloader;
extern const parse_inst_t PROGMEM cmd_log;
extern const parse_inst_t PROGMEM cmd_log_show;
extern const parse_inst_t PROGMEM cmd_log_type;
extern const parse_inst_t PROGMEM cmd_stack_space;
extern const parse_inst_t PROGMEM cmd_callout;

/**********************************************************/

/* this structure is filled when cmd_test_eeprom_config is parsed successfully */
struct cmd_test_eeprom_config_result {
	fixed_string_t arg0;
};

static void cmd_test_eeprom_config_parsed(void *parsed_result, void *data)
{
	(void)parsed_result;
	(void)data;

	eeprom_dump_cmds();
	eeprom_append_cmd("salut1\n");
	eeprom_dump_cmds();
	eeprom_append_cmd("salut2\n");
	eeprom_append_cmd("salut3\n");
	eeprom_append_cmd("salut4\n");
	eeprom_dump_cmds();
	eeprom_insert_cmd_before("coin\n", 0);
	eeprom_insert_cmd_before("coin2\n", 2);
	eeprom_dump_cmds();
	eeprom_delete_cmd(2);
	eeprom_delete_cmd(0);
	eeprom_dump_cmds();
}

const char PROGMEM str_test_eeprom_config_arg0[] = "test_eeprom_config";
const parse_token_string_t PROGMEM cmd_test_eeprom_config_arg0 =
	TOKEN_STRING_INITIALIZER(struct cmd_test_eeprom_config_result, arg0,
				 str_test_eeprom_config_arg0);

const char PROGMEM help_test_eeprom_config[] = "Test the eeprom configuration";
const parse_inst_t PROGMEM cmd_test_eeprom_config = {
	.f = cmd_test_eeprom_config_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_test_eeprom_config,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_test_eeprom_config_arg0,
		NULL,
	},
};

/* ************* */

struct cmd_eeprom_del_result {
	fixed_string_t cmd;
	fixed_string_t action;
	uint8_t n;
};

static void cmd_eeprom_del_parsed(void *parsed_result,
				void *data)
{
	struct cmd_eeprom_del_result *res = parsed_result;

	(void)data;
	if (eeprom_delete_cmd(res->n) < 0)
		printf_P(PSTR("cannot delete command\n"));
	eeprom_dump_cmds();
}

const char PROGMEM str_eeprom_del_eeprom[] = "eeprom";
const parse_token_string_t PROGMEM cmd_eeprom_del_cmd =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_del_result, cmd,
				 str_eeprom_del_eeprom);
const char PROGMEM str_eeprom_del_del[] = "del";
const parse_token_string_t PROGMEM cmd_eeprom_del_action =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_del_result, action,
				 str_eeprom_del_del);
const parse_token_num_t PROGMEM cmd_eeprom_del_num =
	TOKEN_NUM_INITIALIZER(struct cmd_eeprom_del_result, n,
			      UINT8);

const char PROGMEM help_eeprom_del[] = "delete an eeprom init command";
const parse_inst_t PROGMEM cmd_eeprom_del = {
	.f = cmd_eeprom_del_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_eeprom_del,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_eeprom_del_cmd,
		(PGM_P)&cmd_eeprom_del_action,
		(PGM_P)&cmd_eeprom_del_num,
		NULL,
	},
};

/* ************* */

struct cmd_eeprom_add_result {
	fixed_string_t cmd;
	fixed_string_t action;
	uint8_t n;
};

static void cmd_eeprom_add_parsed(void *parsed_result,
				 void *data)
{
	struct cmd_eeprom_add_result *res = parsed_result;
	struct rdline rdl;
	const char *buffer;
	int8_t ret;
	int16_t c;

	rdline_init(&rdl, cmdline_write_char, NULL, NULL);
	rdline_newline(&rdl, "> ");

	while (1) {
		c = cmdline_dev_recv(NULL);
		if (c < 0)
			continue;

		ret = rdline_char_in(&rdl, c);
		if (ret == -2) {
			printf_P(PSTR("abort\n"));
			return;
		}
		if (ret == 1)
			break;
	}

	buffer = rdline_get_buffer(&rdl);
	if (data == NULL)
		eeprom_insert_cmd_before(buffer, res->n);
	else
		eeprom_append_cmd(buffer);
	eeprom_dump_cmds();
}

const char PROGMEM str_eeprom_add_eeprom[] = "eeprom";
const parse_token_string_t PROGMEM cmd_eeprom_add_cmd =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_add_result, cmd,
				 str_eeprom_add_eeprom);
const char PROGMEM str_eeprom_add_add[] = "add";
const parse_token_string_t PROGMEM cmd_eeprom_add_action =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_add_result, action,
				 str_eeprom_add_add);
const parse_token_num_t PROGMEM cmd_eeprom_add_num =
	TOKEN_NUM_INITIALIZER(struct cmd_eeprom_add_result, n,
			      UINT8);

const char PROGMEM help_eeprom_add[] = "insert an eeprom init command";
const parse_inst_t PROGMEM cmd_eeprom_add = {
	.f = cmd_eeprom_add_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_eeprom_add,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_eeprom_add_cmd,
		(PGM_P)&cmd_eeprom_add_action,
		(PGM_P)&cmd_eeprom_add_num,
		NULL,
	},
};

const char PROGMEM help_eeprom_add2[] = "append an eeprom init command";
const parse_inst_t PROGMEM cmd_eeprom_add2 = {
	.f = cmd_eeprom_add_parsed,  /* function to call */
	.data = (void *)1,      /* 2nd arg of func */
	.help_str = help_eeprom_add2,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_eeprom_add_cmd,
		(PGM_P)&cmd_eeprom_add_action,
		NULL,
	},
};

/* ************* */

struct cmd_eeprom_list_result {
	fixed_string_t cmd;
	fixed_string_t action;
};

static void cmd_eeprom_list_parsed(void *parsed_result,
				void *data)
{
	(void)parsed_result;
	(void)data;
	eeprom_dump_cmds();
}

const char PROGMEM str_eeprom_list_eeprom[] = "eeprom";
const parse_token_string_t PROGMEM cmd_eeprom_list_cmd =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_list_result, cmd,
				 str_eeprom_list_eeprom);
const char PROGMEM str_eeprom_list_list[] = "list";
const parse_token_string_t PROGMEM cmd_eeprom_list_action =
	TOKEN_STRING_INITIALIZER(struct cmd_eeprom_list_result, action,
				 str_eeprom_list_list);

const char PROGMEM help_eeprom_list[] = "list all eeprom init commands";
const parse_inst_t PROGMEM cmd_eeprom_list = {
	.f = cmd_eeprom_list_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_eeprom_list,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_eeprom_list_cmd,
		(PGM_P)&cmd_eeprom_list_action,
		NULL,
	},
};


/* ************* */

/* in progmem */
const parse_ctx_t PROGMEM main_ctx[] = {

	/* commands_gen.c */
	&cmd_reset,
	&cmd_bootloader,
	&cmd_log,
	&cmd_log_show,
	&cmd_log_type,
	&cmd_stack_space,
	&cmd_callout,
	&cmd_test_eeprom_config,
	&cmd_eeprom_del,
	&cmd_eeprom_add,
	&cmd_eeprom_add2,
	&cmd_eeprom_list,
	NULL,
};
