/*
 * Copyright (c) 2014, Olivier MATZ <zer0@droids-corp.org>
 * Copyright (c) 2014, Fabrice DESCLAUX <serpilliere@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef IMU_H_
#define IMU_H_

struct imu_info {
	/* gyro */
	double gx;
	double gy;
	double gz;

	/* accelero */
	double ax;
	double ay;
	double az;

	/* magneto */
	double mx;
	double my;
	double mz;

	/* temperature */
	double temp;
};

struct quaternion {
	double q0;
	double q1;
	double q2;
	double q3;
};

struct euler {
	double roll;
	double pitch;
	double yaw;
};

/* initialize the IMU */
void imu_init(void);

/* if sd log file is opened, log the status of IMU on the sdcard */
int imu_log(void);

/* return a filled imu_info structure corresponding to the latest axes read in
 * the timer callback. Does not lock irq, so it's up to the user to do that. */
void imu_get_info(struct imu_info *imu);

/* return the latest position in a quaternion struct read in the timer
 * callback. Does not lock irq, so it's up to the user to do that. */
void imu_get_pos_quat(struct quaternion *pos);

/* return the latest position in an euler struct read in the timer
 * callback. Does not lock irq, so it's up to the user to do that. */
void imu_get_pos_euler(struct euler *pos);

#endif
